﻿namespace Olap.SL
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.Specialized;
    using System.Linq;

    using Infragistics.Olap;
    using Infragistics.Olap.FlatData;

    public class CustomMeasureAggregator : Aggregator<double>
    {
        private FlatDataSource _flatDataSource;
        private IList<IMeasureViewModel> _measures;

        public CustomMeasureAggregator(FlatDataSource flatDataSource)
        {
            this._flatDataSource = flatDataSource;
            this._measures = new List<IMeasureViewModel>();
            this._flatDataSource.Measures.CollectionChanging += this.Measures_CollectionChanging;
        }

        public override IAggregationResult<double, double> Evaluate(IAggregationResult<double, double> oldResult, IAggregateable aggregateable, IEnumerable items)
        {
            double result = 0;
          
            if (this._measures.Count > 0)
            {
                // implement your custom logic here
                // you can get access to the measures stored in the list
                IMeasureViewModel costMeasureViewModel =
                    this._measures.Where(mvm => mvm.Measure.Name == "Cost").FirstOrDefault();
                
                IAggregateable costAggregateable = null;
                if (costMeasureViewModel != null)
                {
                    costAggregateable = costMeasureViewModel.Measure as IAggregateable;                    
                }

                IMeasureViewModel unitsMeasureViewModel =
                    this._measures.Where(mvm => mvm.Measure.Name == "Units").FirstOrDefault();
                IAggregateable unitsAggregateable = null;
                if (unitsMeasureViewModel != null)
                {
                    unitsAggregateable = unitsMeasureViewModel.Measure as IAggregateable;                    
                }
                
                foreach (var item in items)
                {
                    double cost = 0;
                    int units = 0;

                    if (costAggregateable != null)
                    {
                        cost = (double)costAggregateable.GetValue(item);
                    }

                    if (unitsAggregateable != null)
                    {
                        units = (int)(double)unitsAggregateable.GetValue(item);
                    }

                    result += units * cost;
                }
            }

            return new SingleResult<double>(result);
        }

        public override IAggregationResult<double, double> Evaluate(IAggregationResult<double, double> oldResult, double value)
        {
            // you don't need this one
            throw new NotImplementedException();
        }

        private void Measures_CollectionChanging(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.OldItems != null)
            {
                foreach (var newItem in e.OldItems)
                {
                    IMeasureViewModel measureViewModel = newItem as IMeasureViewModel;
                    if (measureViewModel != null)
                    {
                        this._measures.Remove(measureViewModel);
                    }
                }
            }

            if (e.NewItems != null)
            {
                foreach (var newItem in e.NewItems)
                {
                    IMeasureViewModel measureViewModel = newItem as IMeasureViewModel;
                    if (measureViewModel != null)
                    {
                        FlatMeasure measure = (FlatMeasure)measureViewModel.Measure;
                        if (measure.Aggregator != this)
                        {
                            this._measures.Add(measureViewModel);
                        }
                    }
                }
            }
        }
    }
}